<?php
declare(strict_types = 1);

error_reporting(-1);

spl_autoload_register(function(string $class)
{
	require_once('includes/core/' . $class . '.class.php');
});

class l10nText
{
	public static $comments = FALSE;
	private static $_dir;
	private static $_text = [];

	public static function extract(string $dir): void
	{
		if (!self::$_dir)
		{
			self::$_dir = $dir;
		}
		foreach (scandir($dir) as &$f)
		{
			if (in_array($f, ['.', '..']))
			{
				continue;
			}
			if (is_dir($f = $dir . '/' . $f))
			{
				self::extract($f);
				continue;
			}
			if (is_file($f) && substr($f, -4) == '.php')
			{
				self::_getL10nText($f);
			}
		}
	}

	public static function save(): bool
	{
		$text_sort = [];
		$n = 0;
		foreach (self::$_text as $text => &$files)
		{
			$text_sort[strtolower(Utility::removeAccents($text)) . '-' . $n] = $text;
			$n++;
		}
		ksort($text_sort);

		$content = '<?php' . "\n";
		$content .= '$text =' . "\n";
		$content .= '[' . "\n";
		if (self::$comments)
		{
			$content .= "\t" . '// includes/core/L10N.class.php:83' . "\n";
		}
		$dates =
		[
			'Jan', 'January', 'Feb', 'February', 'Mar', 'March', 'Apr', 'April', 'May',
			'Jun', 'June', 'Jul', 'July', 'Aug', 'August', 'Sep', 'September', 'Oct',
			'October', 'Nov', 'November', 'Dec', 'December', 'Mon', 'Monday', 'Tue',
			'Tuesday', 'Wed', 'Wednesday', 'Thu', 'Thursday', 'Fri', 'Friday', 'Sat',
			'Saturday', 'Sun', 'Sunday'
		];
		foreach ($dates as $k => &$d)
		{
			$content .= "\t" . '"' . $d . '" => "",' . "\n";
		}
		foreach ($text_sort as &$text)
		{
			$msgid = str_replace(['"', '$'], ['\\"', '\\$'], $text);
			$msgstr = '';
			if (self::$comments)
			{
				$comments = "\n";
				foreach (self::$_text[$text] as &$file)
				{
					$comments .= "\t" . '// ' . $file . "\n";
				}
				$content .= $comments;
			}
			$content .= "\t" . '"' . $msgid . '" => "' . $msgstr . '",' . "\n";
		}
		$content .= ']' . "\n";
		$content .= '?>';

		return File::putContents(__DIR__ . '/text.php', $content) !== FALSE;
	}

	private static function _getL10nText(string $f): void
	{
		$data = file($f);

		// Récupération des numéros de lignes.
		$lines_l10n = [];
		foreach ($data as $i => &$line)
		{
			preg_match_all('`(__\(\s*\')`U', $line, $m);
			if (count($m[1]) == 0)
			{
				continue;
			}
			$lines_l10n[$i + 1] = count($m[1]);
		}

		if (count($lines_l10n) == 0)
		{
			return;
		}

		$l10n_line = [];
		foreach ($lines_l10n as $line => &$nb_text)
		{
			for ($i = 0; $i < $nb_text; $i++)
			{
				$l10n_line[] = $line;
			}
		}

		$data = implode($data);
		
		// Suppression des concaténations de chaînes.
		$data = preg_replace('`\'(?<!\x5c)\s*\.\s*\'(?<!\x5c)`', '', $data);

		// Suppression des antislashes.
		$data = preg_replace('`(?<!\x5c(?<!\x5c))\x5c\'`', '\'', $data);
		$data = str_replace('\\\\', '\\', $data);

		// Récupération des textes localisés.
		preg_match_all('`__\(\s*\'((?:.(?<![^\x5c]\'\)))+)\'\s*\)`Us', $data, $m);

		foreach ($m[1] as $i => &$t)
		{
			if (!isset(self::$_text[$t]))
			{
				self::$_text[$t] = [];
			}
			self::$_text[$t][] = str_replace(self::$_dir . '/', '', $f) . ':' . $l10n_line[$i];
		}
	}
}
?>
<!DOCTYPE html>
<html lang="fr">

<head>
	<title>Création d'un fichier de langue</title>
	<link href="./template/default/style//style.css" rel="stylesheet">
	<style type="text/css">
	form
	{
		margin: 15px;
	}
	</style>
</head>

<body>

<form action="" method="post">
	<p class="field">
		<input<?php if (isset($_POST['comments'])) : ?> checked<?php endif; ?> type="checkbox" name="comments" id="comments">
		<label for="comments">Ajouter les fichiers et numéros de ligne en commentaires.</label>
	</p>
	<p id="submit">
		<input name="extract" type="submit" class="button" value="Créer le fichier de langue">
	</p>
<?php
if (isset($_POST['extract']))
{
	l10nText::$comments = isset($_POST['comments']);
	l10nText::extract(__DIR__);
	if (l10nText::save())
	{
		echo '<p class="message_success">Le fichier de langue "text.php" a été créé avec succès.</p>';
	}
	else
	{
		echo '<p class="message_error">Impossible de créer le fichier de langue car une erreur s\'est produite.</p>';
	}
}
?>
</form>

</body>

</html>
